/************************************************************************
 ************************************************************************
                                                                         
 Function :                                                              

 Colour bitmapGetColour(Bitmap bitmap, int xPos, int yPos);
 void   bitmapPrint(Bitmap bitmap);
 int    bitmapWidth(Bitmap bitmap);
 int    bitmapHeight(Bitmap bitmap);
 Bitmap bitmapMake(int width, int height);
 Bitmap bitmapSetColour(Bitmap bitmap, Colour colour,int xPos, int yPos);
 
 
 Module description :

 Handles bitmaps that consist of a width, a height and coloured bits.
 
                                                                         
 
                                                                         
*************************************************************************
*************************************************************************/

#ifndef _BITMAP_H
#define _BITMAP_H

#include "bool.h"
#include "colour.h"
#include "scanLine.h"

typedef ScanLine *Row;

/* Bitmap structure */
typedef struct
{
  Row         rows;
  int         width;
  int         height;
} Bitmap;




/*-------------------------------------------------------------
 Function : bitmapMake                                                   
                                                               
 Argument : int,
            int
                                                               
 ReturnType : Bitmap                                                 
                                                               
 Description : Create a new bitmap of size width x height


--------------------------------------------------------------*/
Bitmap bitmapMake(int width, int height);


/*-------------------------------------------------------------
 Function : bitmapDelete                                                   
                                                               
 Argument : Bitmap                                                   
                                                               
 ReturnType : Bool                                                 
                                                               
 Description : Free allocated memory that was occupied by bitmap
 
                                                               
--------------------------------------------------------------*/
void bitmapDelete(Bitmap bitmap);


/*-------------------------------------------------------------
 Function : bitmapSetColour                                                   
                                                               
 Argument : Bitmap,
            Colour,
            int,
            int
                                                               
 ReturnType : Bitmap                                                 
                                                               
 Description : Given a bitmap and a colour, the colour is inserted
               at position xPos - yPos.
                                                               
                                                               
--------------------------------------------------------------*/
Bitmap bitmapSetColour(Bitmap bitmap, Colour colour, int xPos, int yPos);


/*-------------------------------------------------------------
 Function : bitmapGetColour                                                   
                                                               
 Argument : Bitmap,
            int,
            int
            
 ReturnType : Colour                                                 
                                                               
 Description : Select the colour that appears in bitmap                                                
                                                               
                                                               
--------------------------------------------------------------*/
Colour bitmapGetColour(Bitmap bitmap, int xPos, int yPos);

/*-------------------------------------------------------------
 Function : bitmapWidth                                                   
                                                               
 Argument : Bitmap                                                   
                                                               
 ReturnType : int                                                 
                                                               
 Description : From a bitmap, the function returns the width of it.                                                
                                                               
                                                               
--------------------------------------------------------------*/
int bitmapWidth(Bitmap bitmap);


/*-------------------------------------------------------------
 Function : bitmapHeight                                                   
                                                               
 Argument : Bitmap                                                   
                                                               
 ReturnType : int                                                 
                                                               
 Description : From a bitmap, the function returns the height of it.
 
--------------------------------------------------------------*/
int bitmapHeight(Bitmap bitmap);


/*-------------------------------------------------------------
 Function : bitmapRezise                                                    
                                                               
 Argument : Bitmap,
 			int,
 			int
                                                                
 ReturnType : Bitmap                                                
                                                               
 Description : Rezise the bitmap to new size.
--------------------------------------------------------------*/
Bitmap bitmapResize(Bitmap bmp , int width , int height);


Bitmap bitmapSetRow(unsigned char * buffer , Bitmap bmp , int Line);

Bitmap bitmapSetLine(Bitmap bmp , int line , ScanLine scanLine);

ScanLine bitmapGetLine(Bitmap bmp , int line);

Bitmap bitmapResizeCol(Bitmap bitmap , int height , Bitmap bitmap2);


unsigned char * bitmapResizeLine(unsigned char * buffer , 
									int oldWidth , int newWidth);


unsigned char * bitmapGetJsample(Bitmap bmp ,int line);

Bitmap bitmapSetJsample(unsigned char * jsample , Bitmap bmp ,int line);


/*-------------------------------------------------------------
 Function : bitmapPrint                                                    
                                                               
 Argument : bitmap                                                   
                                                               
 ReturnType : void                                                 
                                                               
 Description : Print the bitmaps inserted colour <red, green, blue>                                                              
--------------------------------------------------------------*/
void bitmapPrint(Bitmap bitmap);


#endif /* _BITMAP_H */





