/************************************************************************
 ************************************************************************
                                                                         
 Function : 

 ColourValue colourGetRedValue(Colour colour);
 ColourValue colourGetGreenValue(Colour colour);
 ColourValue colourGetBlueValue(Colour colour);
 ColourValue colourIntToColourValue(int value);
 ColourValue colourFloatToColourValue(float value);
 Colour      colourMake(ColourValue red, ColourValue green, ColourValue blue);
 Colour      colourScale(Colour colour, float percent);
 Colour      colourUnion(Colour colourOne, Colour colourTwo);
 Colour      colourDefaultColour(void);
 Colour      colourSetValue(Colour colour, 
                            ColourValue red, 
			    ColourValue green, 
			    ColourValue blue);



 Test funktion:

void colourPrint(Colour colour);



 Module description : The module keep all the functions that is assosiated to
                      the colour structure.
                                                                         
 		      
*************************************************************************
*************************************************************************/



#ifndef _COLOUR_H
#define _COLOUR_H

typedef int ColourValue;

typedef unsigned int Colour;

typedef struct
{
	int red;
	int green;
	int blue;
}ColourAcc;


/*-------------------------------------------------------------
 Function : colourSetValue

 Argument : Colour,
            ColourValue,
            ColourValue,
            ColourValue,
            ColourValue
                                                   
 ReturnType : Colour                                                  
                                                               
 Description : Set the colour to a colour structure 
               with the colourValue red, green and blue
                                                               
--------------------------------------------------------------*/
Colour colourSetValue(Colour colour, 
		      ColourValue red, 
		      ColourValue green, 
		      ColourValue blue);


/*-------------------------------------------------------------
 Function : colourMake                                                    
                                                               
 Argument : ColourValue,
            ColourValue,
            ColourValue
                                                               
 ReturnType : Colour                                                 
                                                               
 Description : Make a new colour structure where the colour in 
               the object is red,green and blue
                                                               
                                                               
--------------------------------------------------------------*/
Colour colourMake(ColourValue red, ColourValue green, ColourValue blue);


/*-------------------------------------------------------------
 Function : colourRedValue                                                   
                                                               
 Argument : Colour                                                   
                                                               
 ReturnType : ColourValue                                                 
                                                               
 Description : Substract the red colourValue from colour
                                                               
--------------------------------------------------------------*/
ColourValue colourGetRedValue(Colour colour);


/*-------------------------------------------------------------
 Function : colourGetGreenValue
                                                               
 Argument : Colour                                                   
                                                               
 ReturnType : ColourValue                                                 
                                                               
 Description : Substract the green colourValue from colour
                                                               
--------------------------------------------------------------*/
ColourValue colourGetGreenValue(Colour colour);



/*-------------------------------------------------------------
 Function : colourGetBlueValue
                                                               
 Argument : Colour                                                   
                                                               
 ReturnType : ColourValue                                                 
                                                               
 Description : Substract the blue colourValue from colour
 
--------------------------------------------------------------*/
ColourValue colourGetBlueValue(Colour colour);



/*-------------------------------------------------------------
 Function : colourIntToColourValue
                                                               
 Argument : int                                                   
                                                               
 ReturnType : ColourValue                                                 
                                                               
 Description : translate a integer to a colourValue
                                                               
--------------------------------------------------------------*/
ColourValue colourIntToColourValue(int value);

/*-------------------------------------------------------------
 Function : colourFloatToColourValue
                                                               
 Argument : float                                                    
                                                               
 ReturnType : ColourValue                                                 
                                                               
 Description : Translate a float to ColourValue

--------------------------------------------------------------*/
ColourValue colourFloatToColourValue(float value);

/*-------------------------------------------------------------
 Function : colourUnion                                                   
                                                               
 Argument : Colour,
            Colour
                                                               
 ReturnType : Colour                                                 
                                                               
 Description : Union between two colours and return the new
               colour that appears after the uniting.
                                                               
--------------------------------------------------------------*/
ColourAcc colourUnion(ColourAcc colourOne, Colour colourTwo);


/*-------------------------------------------------------------
 Function : colourScale                                                   
                                                               
 Argument : Colour,
            float
                                                               
 ReturnType : Colour                                                 
                                                               
 Description : Scale the colour with X-percent.
 Example:  0.23 will scale down 23% and 1.23 will scale up 123%.
                                                                        
--------------------------------------------------------------*/
Colour colourScale(Colour colour, float percent);

Colour colourAccScale(ColourAcc colour ,float scale);

ColourAcc colourToColourAcc(Colour colour);

/* Test function */

/*-------------------------------------------------------------
 Function : colourPrint                                                    
                                                               
 Argument : Colour                                                   
                                                               
 ReturnType : void                                                 
                                                               
 Description : Just print the 'RGB' values.
                                                               
--------------------------------------------------------------*/
void colourPrint(Colour colour);

#endif /* _COLOUR_H */

