/************************************************************************
 ************************************************************************
                                                                         
 Module : ppmLoad.c                                                                
                                                                         
                                                                         
                                                                         
 Changes :                                                               
                                                                         
                                                                         
************************************************************************* 
*************************************************************************/



#include <stdio.h>
#include <assert.h>

#include "ppmIO.h"
#include "bool.h"
#include "errorHandler.h"
#include "debug.h"
#include "colour.h"
#include "bitmap.h"


/*-------------------------------------------------------------
 Function : isNum                                                    
                                                               
 Purpus :                                                      
                                                               
                                                               
--------------------------------------------------------------*/
static Bool isNum(int ch)
{
  /* Just for debug */
  assert (ch >= 0 && ch <= MAXCHARINT);
  debugPPMLoad("isNum");
  debugExit();
  


  if ((ch >= NOLL) && (ch <= NIO))
    return (TRUE);
  else
    return (FALSE);

}


/*-------------------------------------------------------------
 Function : chewChars                                              
                                                               
 Purpus : Eating whitespaces from an infile.
--------------------------------------------------------------*/
static int chewChars(FILE* file)
{
  /* local variable */
  int   ch;
  Bool  statement = TRUE;

  
  /* Just for debug */
  debugPPMLoad("chewChars");
 


  while (statement)
    {
      ch = fgetc (file);
      switch (ch)
	{
	case EOF:
	  errorHandler("ppmLoad","End of file reached prematurely");
	  statement = FALSE;
	  break;
       	case SQUARE:
	  while (fgetc(file) != NEWLINE);
	  break;
	case SPACE:
	  break;
	case NEWLINE:
	  break;
	case TAB:
	  break;
	case CRETURN:
	  break;
	default:
	  statement = FALSE;
	  break;
	}
    }



  debugExit();
  return ch;
}

/*-------------------------------------------------------------
 Function : getNextNum                                                
                                                               
 Purpus : from a stream (infile) return the next integer
          written in ascii
--------------------------------------------------------------*/
static int getNextNum (FILE *file)
{
  /* local variable */
  int i;
  int ch;
  int value = 0;
  int sofar;


  ch = chewChars (file);

  
  /* Just for debug */
  assert (ch >= 0 && ch <= MAXCHARINT);
  debugPPMLoad("getNextNum");
  


  
  for (i = 0 ; i < 5 ; i++)
    {
      sofar = value;
      
      if (isNum(ch))
	{
	  value = ch - NOLL + (sofar * 10);
	  ch = fgetc (file);
	  if (ch == EOF)
	    errorHandler("ppmLoad","Reached end of file");
	}
      else
	break;
    }

  

  debugExit();
  return (value);
}


    
/*-------------------------------------------------------------
 Function :  asciiToColour                                                  
                                                               
 Purpus : Returns the next colour out of a string from stdin
 
--------------------------------------------------------------*/
static Colour asciiToColour (FILE* file)
{
  /* local variable */
  int         rgbvalue;
  ColourValue red;
  ColourValue green;
  ColourValue blue;
  Colour      col;




  /* Just for debug */
  debugPPMLoad("asciiToColour");
  

  

  rgbvalue = getNextNum(file);
  red      = colourIntToColourValue(rgbvalue);

  rgbvalue = getNextNum(file);
  green    = colourIntToColourValue(rgbvalue);

  rgbvalue = getNextNum(file);
  blue     = colourIntToColourValue(rgbvalue);

  col      = colourMake(red , green , blue);



  
  debugExit();
  return (col);
}

/*-------------------------------------------------------------
 Function :  binaryToColour                                                  
                                                               
 Purpus : Returns the next colour out of a string from stdin
 
--------------------------------------------------------------*/
static Colour binaryToColour (FILE* file)
{
  /* local variable */
  int         rgbvalue;
  ColourValue red;
  ColourValue green;
  ColourValue blue;
  Colour      col;

 


 /* Just for debug */
  debugPPMLoad("binaryToColour");
 




  
  rgbvalue = fgetc(file);
  if (rgbvalue == EOF)
    errorHandler("ppmLoad","Got EOF, expected red colour");
  red = colourIntToColourValue(rgbvalue);
  
  rgbvalue = fgetc(file);
  if (rgbvalue == EOF)
    errorHandler("ppmLoad","Got EOF, expected green colour");
  green = colourIntToColourValue(rgbvalue);
  
  rgbvalue = fgetc(file);
  if (rgbvalue == EOF)
    errorHandler("ppmLoad","Got EOF, expected blue colour");
  blue = colourIntToColourValue(rgbvalue);
      
  col = colourMake(red , green , blue);





  debugExit();
  return (col);
}



/*-------------------------------------------------------------
 Function :  setColourArray
                                                               
 Purpus : Returns the bitmap out of chars coming
          from stdin.
                                                               
--------------------------------------------------------------*/
static Bitmap setColourArray (FILE* file , Bitmap picture , int t)
{
  /* local variable */
  int    w , h;
  int    x , y;
  Colour col;
  


  w = bitmapWidth (picture);
  h = bitmapHeight(picture);



  /* Just for debug */
  assert (w >= 0 &&
	  w <= MAXNUMBERSIZE &&
	  h >= 0 &&
	  h <= MAXNUMBERSIZE);
  debugPPMLoad("getColourArray");



  if (t == 6)
    {
      for (y = 1 ; y <= h ; y++)
	{
	  for (x = 1 ; x <= w ; x++)
	    {
	      col = binaryToColour (file);
	      picture = bitmapSetColour(picture , col , x , y);
	    }
	}
    }
  else
    {
      if (t != 3)
	errorHandler("ppmLoad" , "Unknown ppm-type.");
      for (y = 1 ; y <= h ; y++)
	{
	  for (x = 1 ; x <= w ; x++)
	    {
	      col = asciiToColour (file);
	      picture = bitmapSetColour(picture , col , x , y);
	    }
	}
    } 



  
  debugExit();
  return (picture);
}


/*-------------------------------------------------------------
 Function : ppmIOLoad                                             
                                                               
 Purpus : Taking a ppm infile and returning the corresponding
          bitmap as declared in bitmap.h
--------------------------------------------------------------*/
Bitmap ppmIOLoad (char* name)
{
  /* local variable */  
  Bitmap pic;
  int    w , h, t;
  FILE*  file;




  /* Just for debug */
  debugPPMLoad("ppmIOLoad");
 

  
  
  file = fopen(name, "r");
  if (file == NULL)
    errorHandler("ppmLoad","Failed to open ppm-file");


  /* getting filedescription */
  t = chewChars (file);

  if (t != (int)'P')
    errorHandler("ppmLoad" , "File most likely not a ppm");


  t = getNextNum(file);
  w = getNextNum(file);
  h = getNextNum(file);

  /* dummy to remove the colourdepth number before
     reading the actual colours */
  (void)getNextNum(file);
  


  pic = bitmapMake(w , h);
  pic = setColourArray(file , pic , t);




  debugExit();
  return(pic);
}







