#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "ppmIO.h"
#include "bool.h"
#include "errorHandler.h"
#include "debug.h"
#include "colour.h"
#include "bitmap.h"


static char ASCIICOLOUR[12];
static char ASCIICOLVAL[4];


/*-------------------------------------------------------------
 Function : writeToFile
                                                               
 Purpus : Writing the string to a file and handle possible
          errors
--------------------------------------------------------------*/
static void writeToFile(char *string , FILE* file)
{
  /* local variable */
  int errcode;

  errcode = fputs(string , file);
  if (errcode == EOF)
    errorHandler("ppmSave","unable to write to file");

  return;
}


/*-------------------------------------------------------------
 Function : intToAscii                                               
                                                               
 Purpus : converting an integer to a text string
          integer must be at the most numstring characters
	  long.
	  Fills out the leading characters with spaces.
--------------------------------------------------------------*/
static char* intToAscii(int number , char* numstring)
{
  /* local variable */
  int a, i , j;

  /*Just for debug */
  assert (number < MAXNUMBERSIZE);
  debugPPMSave ("intToAscii");
 
  
  i = strlen(numstring);
  printf("%d\n",i);
  j = i - 1;

  
  while(i > 0)
    {
      i--;
      a = number / 10;
      numstring[i] = (char)(number - (a * 10) + NOLL);
      number = a;
    }
  
    
  while ((numstring[i] == NOLL) && (i < j))
    {
      numstring[i] = ' ';
      i++;
    }

  debugExit();
  return numstring;
}



/*-------------------------------------------------------------
 Function : colorToAscii
 
 Purpus : Takes a 12 character string and a colour.
          Inserts the ascii integer after eachother.
                                                               
--------------------------------------------------------------*/
static void colourToAscii(Colour col)
{
  /* local variable */
  ColourValue c;
  
  /* Just for debug */
  /*colourGetColValue can't return faulty values.
  assert (colourGetRedValue(col) >= 0 &&
	  colourGetRedValue(col) <= MAXCHARINT &&
	  colourGetGreenValue(col) >= 0 &&
	  colourGetGreenValue(col) <= MAXCHARINT &&
	  colourGetBlueValue(col) >= 0 &&
	  colourGetBlueValue(col) <= MAXCHARINT);
	  */
  debugPPMSave ("colourToAscii");
 

  
  c = (colourGetRedValue(col));
  strcpy(ASCIICOLVAL , intToAscii(c , ASCIICOLVAL));
  strcat(ASCIICOLOUR , ASCIICOLVAL);
  
  c = (colourGetGreenValue(col));
  strcpy(ASCIICOLVAL , intToAscii(c , ASCIICOLVAL));
  strcat(ASCIICOLOUR , ASCIICOLVAL);

  c = (colourGetBlueValue(col));
  strcpy(ASCIICOLVAL , intToAscii(c , ASCIICOLVAL));
  strcat(ASCIICOLOUR , ASCIICOLVAL);


  
  debugExit();
  return;
}


/*-------------------------------------------------------------
 Function : writeBinary
 
 Purpus : Takes a colour and a outputfile;
          writes the rgb-values after
	  eachother and returns void
--------------------------------------------------------------*/
static void writeBinary(Colour col , FILE* file)
{
  /* local variable */
  int r , g , b, errcode;
  
  /* Just for debug */
  /* assert can't fail due to unsigned char as colourvalue
  assert (colourGetRedValue(col) >= 0 &&
	  colourGetRedValue(col) <= MAXCHARINT &&
	  colourGetGreenValue(col) >= 0 &&
	  colourGetGreenValue(col) <= MAXCHARINT &&
	  colourGetBlueValue(col) >= 0 &&
	  colourGetBlueValue(col) <= MAXCHARINT);
  errorHandler("ppmSave","writeBinary");
  */

  
  r = (colourGetRedValue(col));
  errcode = fputc (r , file);
  if ((errcode == EOF))
     errorHandler("ppmSave","Writing red colourValue failed");

  g = (colourGetGreenValue(col));
  errcode = fputc (g , file);
  if ((errcode == EOF))
     errorHandler("ppmSave","Writing green colourValue failed");

  b = (colourGetBlueValue(col));
  errcode = fputc (b , file);
  if ((errcode == EOF))
     errorHandler("ppmSave","Writing blue colourValue failed");


  debugExit();
  return;
}

/*-------------------------------------------------------------
 Function : ppmIOSave                                              
                                                               
 Purpus : Main function in this file, the interface to the user
          Takes a bitmap and a filename and
	  the filetype(t for text, b for binary)
	  and writes it with ppm enconding to disk.
--------------------------------------------------------------*/
Bool ppmIOSave (Bitmap picture , char* filename , char type)
{
  /* local variable */
  Colour col;
  FILE   *file;
  int    i , j;
  int    b , h;
  int    linemaker , errcode;
  Bool   status = FALSE;

 
  
  b = bitmapWidth(picture);
  h = bitmapHeight(picture);
  
  /* Just for debug */
  assert (b > 0 &&
	  b < MAXNUMBERSIZE &&
	  h > 0 &&
	  h < MAXNUMBERSIZE);
  

  
  file = fopen(filename , "w");
  if (file == NULL)
    errorHandler("ppmSave" , "Could not open a file for writing");

  /*Here starts the code for binary type ppm.*/
  /*header is as for ascii*/
  if (type == 'b')
    {
      /* write the filetype. */
      writeToFile ("P6\n" , file);
      writeToFile (AUTHOR , file);

      /*write width and height to file*/
	  fprintf(file, "%d %d\n" , b , h);


      /*write colourdepth*/
      fprintf(file , "%d\n" , 255);/*colourdepth*/


      for (j = 1 ; j <= h ; j++) /*one line at a time*/
		{
		  for (i = 1 ; i <= b ; i++) /*one pixel at a time*/
	    	{
	    	  /*just get the colour and write it char by char
			character can be null hence string writing
			doesn't work*/
	    	  col = bitmapGetColour (picture , i , j);
	  	      writeBinary (col , file);
	    	}
		}
	}

  
  /*Here starts the code for writing the ascii type ppm*/
  /*1. write the filetype.
    2. write who authored the ppm-output.
    3. write the size in width * height.
    4. write the colourdepth.*/
  if (type == 't')
    {
      /* write the filetype. */
      writeToFile ("P3\n" , file);
      writeToFile (AUTHOR , file);

      /*write width and height to file*/
	fprintf(file, "%d %d" , b , h);


      /*write colourdepth*/
      fprintf(file , "%d\n" , 255);/*colourdepth*/
      

	for (j = 1 ; j <= h ; j++) /*lines*/
		{
	  	for (i = 1 ; i < b ; i++) /*bitwise*/
	    	{
	      	linemaker = i + 5;
	      	if (linemaker > b)
				linemaker = b;
	      	for (i = i ; i <= linemaker ; i++)
			/* get five colours (60 chars) at a time then write the
		   	  	ascii-output followed by a newline*/
				{
		  			col = bitmapGetColour (picture , i , j);
		  			colourToAscii (col);
		  			writeToFile (ASCIICOLOUR , file);
				}
	      	strcpy(ASCIICOLOUR , "");
	      	errcode = fputc (NEWLINE , file);
	      	if (errcode == EOF)
				errorHandler("ppmSave","Shit happened during filewrite");
	      	i--;
	    	}
		}
    }
    

  errcode = fclose(file);
  if (errcode == EOF)
    errorHandler("ppmSave" , "Unable to close filestream");


   
  debugExit();
  return status;
}
